------------------------------------------------------------------------------
--  This file is a part of the LPP VHDL IP LIBRARY
--  Copyright (C) 2009 - 2010, Laboratory of Plasmas Physic - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more Cdetails.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
-------------------------------------------------------------------------------
--                    Author : Paul LEROY
--                    Mail   : paul.leroy@lpp.polytechnique.fr
-------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.numeric_std.ALL;
USE IEEE.std_logic_1164.ALL;

LIBRARY staging_lpp;
--USE lpp.general_purpose.ALL;
use staging_lpp.PLE_general_purpose.all;
use staging_lpp.PLE_lpp_fft.all;

ENTITY BUTTERFLY_CTRL IS
  PORT (
    rstn : IN STD_LOGIC;
    clk  : IN STD_LOGIC;

    sample_in_val  : IN  STD_LOGIC;
    sample_out_val : OUT STD_LOGIC;

    sel_op1        : OUT STD_LOGIC_VECTOR( 4 DOWNTO 0 ); -- Are Aim X Y Z
    sel_op2        : OUT STD_LOGIC_VECTOR( 4 DOWNTO 0 ); -- Bre Bim C_in cps_in cms_in
    sel_xyz        : OUT STD_LOGIC_VECTOR( 2 DOWNTO 0 ); -- X Y Z
    sel_out        : OUT STD_LOGIC_VECTOR( 4 DOWNTO 0 );
    alu_ctrl       : OUT STD_LOGIC_VECTOR( 2 DOWNTO 0 );
    alu_comp       : OUT STD_LOGIC_VECTOR( 1 DOWNTO 0 )
    );
END BUTTERFLY_CTRL;

ARCHITECTURE ar_BUTTERFLY_CTRL OF BUTTERFLY_CTRL IS

    TYPE fsm_BUTTERFLY_CTRL_T IS ( clearMAC,
                            waiting,
                            add1,
                            add2,
                            add3,
                            add4,
                            mult5,
                            mac6,
                            mac7,
                            mult8,
                            mac9,
                            last10,
                            last11);
    SIGNAL BUTTERFLY_CTRL_STATE : fsm_BUTTERFLY_CTRL_T;
  
BEGIN

PROCESS (clk, rstn)

    BEGIN  -- PROCESS
        IF rstn = '0' THEN                  -- asynchronous reset (active low)
            --REG -------------------------------------------------------------------
            sel_xyz            <= (OTHERS => '0');
            sel_out            <= (OTHERS => '0');
            --ALU -------------------------------------------------------------------
            sel_op1            <= (OTHERS => '0');      
            sel_op2            <= (OTHERS => '0');
            alu_ctrl           <= ctrl_IDLE;
            alu_comp           <= (OTHERS => '0');
            --OUT
            sample_out_val     <= '0';
      
            BUTTERFLY_CTRL_STATE <= clearMAC;
      
        ELSIF clk'EVENT AND clk = '1' THEN  -- rising clock edge
    
            CASE BUTTERFLY_CTRL_STATE IS

            WHEN clearMAC =>
                IF sample_in_val = '1' THEN
                    alu_ctrl           <= ctrl_CLRMAC;
                    BUTTERFLY_CTRL_STATE  <= waiting;
                END IF;

            WHEN waiting =>
                sel_op1            <= "10000";  -- Are
                sel_op2            <= "10000";  -- Bre
                alu_comp           <= "10";
                BUTTERFLY_CTRL_STATE  <= add1;

            WHEN add1 =>
                sample_out_val     <= '0';
                sel_op1            <= "01000";  -- Aim
                sel_op2            <= "01000";  -- Bim
                alu_comp           <= "10";
                alu_ctrl           <= ctrl_ADD;
                sel_out            <= "10000";
                BUTTERFLY_CTRL_STATE     <= add2;

            WHEN add2 =>
                sample_out_val     <= '0';
                sel_op1            <= "10000";  -- Are
                sel_op2            <= "10000";  -- Bre
                alu_comp           <= "00";
                alu_ctrl           <= ctrl_ADD;
                sel_out            <= "10000";
                BUTTERFLY_CTRL_STATE     <= add3;

            WHEN add3 =>
                sample_out_val     <= '0';
                sel_op1            <= "01000";  -- Aim
                sel_op2            <= "01000";  -- Bim
                alu_comp           <= "00";
                alu_ctrl           <= ctrl_ADD;
                sel_out            <= "10000";
                sel_xyz            <= "100";    -- X
                BUTTERFLY_CTRL_STATE     <= add4;

            WHEN add4 =>
                sample_out_val     <= '0';
                sel_op1            <= "00100";  -- X
                sel_op2            <= "00100";  -- c
                alu_comp           <= "00";
                alu_ctrl           <= ctrl_ADD;
                sel_out            <= "00000";
                sel_xyz            <= "010";
                BUTTERFLY_CTRL_STATE     <= mult5;

            WHEN mult5 =>
                sample_out_val     <= '0';
                alu_ctrl           <= ctrl_MULT;
                sel_op1            <= "00010";  -- Y
                sel_op2            <= "00100";  -- c
                alu_comp           <= "10";
                sel_out            <= "00100";
                BUTTERFLY_CTRL_STATE     <= mac6;

            WHEN mac6 =>
                sample_out_val     <= '0';
                sel_op1            <= "00010";  -- Y
                sel_op2            <= "00001";  -- cms
                alu_comp           <= "00";
                alu_ctrl           <= ctrl_MAC;
                sel_out            <= "10000";
                BUTTERFLY_CTRL_STATE     <= mac7;

            WHEN mac7 =>
                sample_out_val     <= '0';
                sel_op1            <= "00100";  -- X
                sel_op2            <= "00010";  -- cps
                alu_ctrl           <= ctrl_MAC;
                alu_comp           <= "00";
                sel_out            <= "10000";
                BUTTERFLY_CTRL_STATE     <= mult8;

            WHEN mult8 =>
                sample_out_val     <= '0';
                alu_ctrl           <= ctrl_MULT;
                sel_op1            <= "00000";  -- Z is taken directly from the output of the ALU
                sel_op2            <= "00000";  -- 1
                alu_comp           <= "00";
                sel_out            <= "10000";
                BUTTERFLY_CTRL_STATE     <= mac9;

            WHEN mac9 =>
                sample_out_val     <= '0';
                sel_op1            <= "10000";
                sel_op2            <= "10000";
                alu_ctrl           <= ctrl_MAC;
                alu_comp           <= "10";
                sel_out            <= "10000";
                BUTTERFLY_CTRL_STATE     <= last10;

            WHEN last10 =>
                sample_out_val     <= '0';
                sel_op1            <= "10000";
                sel_op2            <= "10000";
                alu_ctrl           <= ctrl_IDLE;
                alu_comp           <= "10";
                sel_out            <= "10000";
                BUTTERFLY_CTRL_STATE     <= last11;

            WHEN last11 =>
                sample_out_val     <= '0';
                alu_ctrl           <= ctrl_IDLE;
                alu_comp           <= "10";
                sel_out            <= "10000";
                BUTTERFLY_CTRL_STATE     <= waiting;

            WHEN OTHERS =>
                NULL;
            
            END CASE;

        END IF;

    END PROCESS;

END ar_BUTTERFLY_CTRL;